@@ -4,7 +4,7 @@
  * (currently mule internal code (mic) is used)
  * Tatsuo Ishii
  *
- * $Header: /cvsroot/pgsql/src/backend/utils/mb/mbutils.c,v 1.36.2.3 2006/05/21 20:07:11 tgl Exp $
+ * $Header: /cvsroot/pgsql/src/backend/utils/mb/mbutils.c,v 1.44 2003/09/25 06:58:05 petere Exp $
  */
 #include "postgres.h"
 
@@ -35,108 +35,162 @@ static pg_enc2name *DatabaseEncoding = &pg_enc2name_tbl[PG_SQL_ASCII];
 static FmgrInfo *ToServerConvProc = NULL;
 static FmgrInfo *ToClientConvProc = NULL;
 
+/*
+ * During backend startup we can't set client encoding because we (a)
+ * can't look up the conversion functions, and (b) may not know the database
+ * encoding yet either.  So SetClientEncoding() just accepts anything and
+ * remembers it for InitializeClientEncoding() to apply later.
+ */
+static bool backend_startup_complete = false;
+static int	pending_client_encoding = PG_SQL_ASCII;
+
+
 /* Internal functions */
 static unsigned char *perform_default_encoding_conversion(unsigned char *src,
 									int len, bool is_client_to_server);
-static int cliplen(const unsigned char *str, int len, int limit);
+static int	cliplen(const unsigned char *str, int len, int limit);
 
-/* Flag to we need to initialize client encoding info */
-static int need_to_init_client_encoding = -1;
 
 /*
- * Set the client encoding and save fmgrinfo for the converion
- * function if necessary. if encoding conversion between client/server
- * encoding is not supported, returns -1
-*/
+ * Set the client encoding and save fmgrinfo for the conversion
+ * function if necessary.  Returns 0 if okay, -1 if not (bad encoding
+ * or can't support conversion)
+ */
 int
 SetClientEncoding(int encoding, bool doit)
 {
 	int			current_server_encoding;
 	Oid			to_server_proc,
 				to_client_proc;
-	FmgrInfo   *to_server = NULL;
-	FmgrInfo   *to_client = NULL;
+	FmgrInfo   *to_server;
+	FmgrInfo   *to_client;
 	MemoryContext oldcontext;
 
-	current_server_encoding = GetDatabaseEncoding();
-
 	if (!PG_VALID_FE_ENCODING(encoding))
 		return (-1);
 
-	/* If we cannot actualy set client encoding info, remeber it
-	 * so that we could set it using InitializeClientEncoding()
-	 * in InitPostgres()
-	 */
-	if (current_server_encoding != encoding && !IsTransactionState())
-		need_to_init_client_encoding = encoding;
-
-	if (current_server_encoding == encoding ||
-	(current_server_encoding == PG_SQL_ASCII || encoding == PG_SQL_ASCII))
+	/* Can't do anything during startup, per notes above */
+	if (!backend_startup_complete)
 	{
-		ClientEncoding = &pg_enc2name_tbl[encoding];
+		if (doit)
+			pending_client_encoding = encoding;
+		return 0;
+	}
+
+	current_server_encoding = GetDatabaseEncoding();
+
+	/*
+	 * Check for cases that require no conversion function.
+	 */
+	if (current_server_encoding == encoding ||
+		(current_server_encoding == PG_SQL_ASCII ||
+		 encoding == PG_SQL_ASCII))
+	{
+		if (doit)
+		{
+			ClientEncoding = &pg_enc2name_tbl[encoding];
+
+			if (ToServerConvProc != NULL)
+			{
+				if (ToServerConvProc->fn_extra)
+					pfree(ToServerConvProc->fn_extra);
+				pfree(ToServerConvProc);
+			}
+			ToServerConvProc = NULL;
+
+			if (ToClientConvProc != NULL)
+			{
+				if (ToClientConvProc->fn_extra)
+					pfree(ToClientConvProc->fn_extra);
+				pfree(ToClientConvProc);
+			}
+			ToClientConvProc = NULL;
+		}
 		return 0;
 	}
 
 	/*
-	 * XXX We cannot use FindDefaultConversionProc() while in bootstrap or
-	 * initprocessing mode since namespace functions will not work.
+	 * If we're not inside a transaction then we can't do catalog lookups,
+	 * so fail.  After backend startup, this could only happen if we are
+	 * re-reading postgresql.conf due to SIGHUP --- so basically this just
+	 * constrains the ability to change client_encoding on the fly from
+	 * postgresql.conf.  Which would probably be a stupid thing to do
+	 * anyway.
 	 */
-	if (IsTransactionState())
-	{
-		to_server_proc = FindDefaultConversionProc(encoding, current_server_encoding);
-		to_client_proc = FindDefaultConversionProc(current_server_encoding, encoding);
+	if (!IsTransactionState())
+		return -1;
 
-		if (!OidIsValid(to_server_proc) || !OidIsValid(to_client_proc))
-			return -1;
-
-		/*
-		 * load the fmgr info into TopMemoryContext so that it survives
-		 * outside transaction.
-		 */
-		oldcontext = MemoryContextSwitchTo(TopMemoryContext);
-		to_server = palloc(sizeof(FmgrInfo));
-		to_client = palloc(sizeof(FmgrInfo));
-		fmgr_info(to_server_proc, to_server);
-		fmgr_info(to_client_proc, to_client);
-		MemoryContextSwitchTo(oldcontext);
-	}
+	/*
+	 * Look up the conversion functions.
+	 */
+	to_server_proc = FindDefaultConversionProc(encoding,
+											   current_server_encoding);
+	if (!OidIsValid(to_server_proc))
+		return -1;
+	to_client_proc = FindDefaultConversionProc(current_server_encoding,
+											   encoding);
+	if (!OidIsValid(to_client_proc))
+		return -1;
 
+	/*
+	 * Done if not wanting to actually apply setting.
+	 */
 	if (!doit)
 		return 0;
 
-	if (IsTransactionState())
+	/*
+	 * load the fmgr info into TopMemoryContext so that it survives
+	 * outside transaction.
+	 */
+	oldcontext = MemoryContextSwitchTo(TopMemoryContext);
+	to_server = palloc(sizeof(FmgrInfo));
+	to_client = palloc(sizeof(FmgrInfo));
+	fmgr_info(to_server_proc, to_server);
+	fmgr_info(to_client_proc, to_client);
+	MemoryContextSwitchTo(oldcontext);
+
+	ClientEncoding = &pg_enc2name_tbl[encoding];
+
+	if (ToServerConvProc != NULL)
 	{
-		ClientEncoding = &pg_enc2name_tbl[encoding];
-
-		if (ToServerConvProc != NULL)
-		{
-			if (ToServerConvProc->fn_extra)
-				pfree(ToServerConvProc->fn_extra);
-			pfree(ToServerConvProc);
-		}
-		ToServerConvProc = to_server;
-
-		if (ToClientConvProc != NULL)
-		{
-			if (ToClientConvProc->fn_extra)
-				pfree(ToClientConvProc->fn_extra);
-			pfree(ToClientConvProc);
-		}
-		ToClientConvProc = to_client;
+		if (ToServerConvProc->fn_extra)
+			pfree(ToServerConvProc->fn_extra);
+		pfree(ToServerConvProc);
 	}
+	ToServerConvProc = to_server;
+
+	if (ToClientConvProc != NULL)
+	{
+		if (ToClientConvProc->fn_extra)
+			pfree(ToClientConvProc->fn_extra);
+		pfree(ToClientConvProc);
+	}
+	ToClientConvProc = to_client;
+
 	return 0;
 }
 
-/* Initialize client encoding if necessary.
- * called from InitPostgres() once during backend starting up.
+/*
+ * Initialize client encoding if necessary.
+ *		called from InitPostgres() once during backend starting up.
  */
 void
-InitializeClientEncoding()
+InitializeClientEncoding(void)
 {
-	if (need_to_init_client_encoding > 0)
+	Assert(!backend_startup_complete);
+	backend_startup_complete = true;
+
+	if (SetClientEncoding(pending_client_encoding, true) < 0)
 	{
-		SetClientEncoding(need_to_init_client_encoding, 1);
-		need_to_init_client_encoding = -1;
+		/*
+		 * Oops, the requested conversion is not available. We couldn't
+		 * fail before, but we can now.
+		 */
+		ereport(FATAL,
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+				 errmsg("conversion between %s and %s is not supported",
+						pg_enc2name_tbl[pending_client_encoding].name,
+						GetDatabaseEncodingName())));
 	}
 }
 
@@ -167,7 +221,7 @@ pg_get_client_encoding_name(void)
  * warn and returns src. We cannot raise an error, since it will cause
  * an infinit loop in error message sending.
  *
- * In the case of no coversion, src is returned.
+ * In the case of no conversion, src is returned.
  *
  * XXX We assume that storage for converted result is 4-to-1 growth in
  * the worst case. The rate for currently supported encoding pares are within 3
@@ -196,23 +250,24 @@ pg_do_encoding_conversion(unsigned char *src, int len,
 	proc = FindDefaultConversionProc(src_encoding, dest_encoding);
 	if (!OidIsValid(proc))
 	{
-		elog(LOG, "default conversion proc for %s to %s not found",
-			 pg_encoding_to_char(src_encoding), pg_encoding_to_char(dest_encoding));
+		ereport(LOG,
+				(errcode(ERRCODE_UNDEFINED_FUNCTION),
+			errmsg("default conversion function for encoding \"%s\" to \"%s\" does not exist",
+				   pg_encoding_to_char(src_encoding),
+				   pg_encoding_to_char(dest_encoding))));
 		return src;
 	}
 
 	/*
-	 * XXX we shoud avoid throwing errors in OidFuctionCall. Otherwise we
-	 * are going into inifinite loop!  So we have to make sure that the
+	 * XXX we should avoid throwing errors in OidFunctionCall. Otherwise
+	 * we are going into infinite loop!  So we have to make sure that the
 	 * function exists before calling OidFunctionCall.
 	 */
 	if (!SearchSysCacheExists(PROCOID,
 							  ObjectIdGetDatum(proc),
 							  0, 0, 0))
 	{
-		elog(LOG, "default conversion proc %u for %s to %s not found in pg_proc",
-			 proc,
-			 pg_encoding_to_char(src_encoding), pg_encoding_to_char(dest_encoding));
+		elog(LOG, "cache lookup failed for function %u", proc);
 		return src;
 	}
 
@@ -269,9 +324,15 @@ pg_convert2(PG_FUNCTION_ARGS)
 	int			len;
 
 	if (src_encoding < 0)
-		elog(ERROR, "Invalid source encoding name %s", src_encoding_name);
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+				 errmsg("invalid source encoding name \"%s\"",
+						src_encoding_name)));
 	if (dest_encoding < 0)
-		elog(ERROR, "Invalid destination encoding name %s", dest_encoding_name);
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+				 errmsg("invalid destination encoding name \"%s\"",
+						dest_encoding_name)));
 
 	/* make sure that source string is null terminated */
 	len = VARSIZE(string) - VARHDRSZ;
@@ -281,10 +342,10 @@ pg_convert2(PG_FUNCTION_ARGS)
 
 	result = pg_do_encoding_conversion(str, len, src_encoding, dest_encoding);
 	if (result == NULL)
-		elog(ERROR, "Encoding conversion failed");
+		elog(ERROR, "encoding conversion failed");
 
 	/*
-	 * build text data type structre. we cannot use textin() here, since
+	 * build text data type structure. we cannot use textin() here, since
 	 * textin assumes that input string encoding is same as database
 	 * encoding.
 	 */
@@ -312,49 +373,9 @@ pg_client_to_server(unsigned char *s, int len)
 	Assert(DatabaseEncoding);
 	Assert(ClientEncoding);
 
-	if (len <= 0)
+	if (ClientEncoding->encoding == DatabaseEncoding->encoding)
 		return s;
 
-	if (ClientEncoding->encoding == DatabaseEncoding->encoding ||
-		ClientEncoding->encoding == PG_SQL_ASCII)
-	{
-		/*
-		 * No conversion is needed, but we must still validate the data.
-		 */
-		(void) pg_verify_mbstr(DatabaseEncoding->encoding, s, len, false);
-		return s;
-	}
-
-	if (DatabaseEncoding->encoding == PG_SQL_ASCII)
-	{
-		/*
-		 * No conversion is possible, but we must still validate the data,
-		 * because the client-side code might have done string escaping
-		 * using the selected client_encoding.  If the client encoding is
-		 * ASCII-safe then we just do a straight validation under that
-		 * encoding.  For an ASCII-unsafe encoding we have a problem:
-		 * we dare not pass such data to the parser but we have no way
-		 * to convert it.  We compromise by rejecting the data if it
-		 * contains any non-ASCII characters.
-		 */
-		if (PG_VALID_BE_ENCODING(ClientEncoding->encoding))
-			(void) pg_verify_mbstr(ClientEncoding->encoding, s, len, false);
-		else
-		{
-			int		i;
-
-			for (i = 0; i < len; i++)
-			{
-				if (s[i] == '\0' || IS_HIGHBIT_SET(s[i]))
-					elog(ERROR,
-						 "invalid byte value for encoding \"%s\": 0x%02x",
-						 pg_enc2name_tbl[PG_SQL_ASCII].name,
-						 (unsigned char) s[i]);
-			}
-		}
-		return s;
-	}
-
 	return perform_default_encoding_conversion(s, len, true);
 }
 
@@ -367,18 +388,8 @@ pg_server_to_client(unsigned char *s, int len)
 	Assert(DatabaseEncoding);
 	Assert(ClientEncoding);
 
-#ifdef USE_REPLICATION
-	if (PGR_Is_Replicated_Query)
+	if (ClientEncoding->encoding == DatabaseEncoding->encoding)
 		return s;
-#endif /* USE_REPLICATION */
-
-	if (len <= 0)
-		return s;
-
-	if (ClientEncoding->encoding == DatabaseEncoding->encoding ||
-		ClientEncoding->encoding == PG_SQL_ASCII ||
-		DatabaseEncoding->encoding == PG_SQL_ASCII)
-		return s;		/* assume data is valid */
 
 	return perform_default_encoding_conversion(s, len, false);
 }
@@ -398,6 +409,16 @@ perform_default_encoding_conversion(unsigned char *src, int len, bool is_client_
 				dest_encoding;
 	FmgrInfo   *flinfo;
 
+#ifdef USE_REPLICATION
+	if ((PGCluster_Info != NULL) &&
+		(PGCluster_Info->PGR_Is_Replicated_Query))
+	{
+		return src;
+	}
+#endif /* USE_REPLICATION */
+	if (len <= 0)
+		return src;
+
 	if (is_client_to_server)
 	{
 		src_encoding = ClientEncoding->encoding;
@@ -414,6 +435,12 @@ perform_default_encoding_conversion(unsigned char *src, int len, bool is_client_
 	if (flinfo == NULL)
 		return src;
 
+	if (src_encoding == dest_encoding)
+		return src;
+
+	if (src_encoding == PG_SQL_ASCII || dest_encoding == PG_SQL_ASCII)
+		return src;
+
 	result = palloc(len * 4 + 1);
 
 	FunctionCall5(flinfo,
@@ -543,7 +570,7 @@ void
 SetDatabaseEncoding(int encoding)
 {
 	if (!PG_VALID_BE_ENCODING(encoding))
-		elog(ERROR, "SetDatabaseEncoding(): invalid database encoding");
+		elog(ERROR, "invalid database encoding");
 
 	DatabaseEncoding = &pg_enc2name_tbl[encoding];
 	Assert(DatabaseEncoding->encoding == encoding);
