@@ -734,46 +734,74 @@ recovery_rsync(char * src , char * dest, int stage)
 	}
 
 	/* recovery step of "global" directory */
+	fprintf(stderr,"%s recovery step of [global] directory...",
+			((stage == 1)?"1st":"2nd"));
 	if (rsync_global_dir(src, dest) != STATUS_OK)
 	{
+		fprintf(stderr,"NG\n");
 		return STATUS_ERROR;
 	}
+	fprintf(stderr,"OK\n");
 
 	if (stage == PGR_1ST_RECOVERY)
 	{
 		/* 1st recovery step of "base" directory */
+		fprintf(stderr,"1st recovery step of [base] directory...");
 		if (first_recovery(src,dest,"base") != STATUS_OK)
 		{
+			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
+		fprintf(stderr,"OK\n");
+
+		fprintf(stderr,"1st recovery step of [pg_clog] directory...");
 		/* 1st recovery step of "pg_clog" directory */
 		if (first_recovery(src,dest,"pg_clog") != STATUS_OK)
 		{
+			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
+		fprintf(stderr,"OK\n");
+
+		fprintf(stderr,"1st recovery step of [pg_xlog] directory...");
 		/* 1st recovery step of "pg_xlog" directory */
 		if (first_recovery(src,dest,"pg_xlog") != STATUS_OK)
 		{
+			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
+		fprintf(stderr,"OK\n");
+
 	}
 	else
 	{
 		/* 2nd recovery step of "base" directory */
+		fprintf(stderr,"2nd recovery step of [base] directory...");
 		if (second_recovery(src,dest,"base") != STATUS_OK)
 		{
+			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
+		fprintf(stderr,"OK\n");
+
 		/* 2nd recovery step of "pg_clog" directory */
+		fprintf(stderr,"2nd recovery step of [pg_clog] directory...");
 		if (second_recovery(src,dest,"pg_clog") != STATUS_OK)
 		{
+			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
+		fprintf(stderr,"OK\n");
+
 		/* 2nd recovery step of "pg_xlog" directory */
+		fprintf(stderr,"2nd recovery step of [pg_xlog] directory...");
 		if (second_recovery(src,dest,"pg_xlog") != STATUS_OK)
 		{
+			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
+		fprintf(stderr,"OK\n");
+
 	}
 
 	return STATUS_OK;
