@@ -22,6 +22,10 @@
  * is" without express or implied warranty.
  *
 */
+#ifdef USE_REPLICATION
+
+#include "postgres.h"
+
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/time.h>
@@ -100,7 +104,7 @@ POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 
 		if (state == 0 && connection_reuse)
 		{
-			int st = 0;
+			int st;
 
 			/* send query for resetting connection such as "ROLLBACK" "RESET ALL"... */
 			st = reset_backend(backend, qcnt);
@@ -489,7 +493,7 @@ static POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 {
 #ifdef PRINT_DEBUG
 	char * func = "ReadyForQuery()";
-#endif			
+#endif
 
 	pool_flush(frontend);
 
@@ -1024,13 +1028,13 @@ static POOL_STATUS CopyDataRows(POOL_CONNECTION *frontend,
 {
 #ifdef PRINT_DEBUG
 	char * func = "CopyDataRows()";
-#endif			
+#endif
 	char *string;
 	int len;
 
 #ifdef PRINT_DEBUG
 	int i = 0;
-	char *buf;
+	char buf[1024];
 #endif
 
 	for (;;)
@@ -1040,14 +1044,11 @@ static POOL_STATUS CopyDataRows(POOL_CONNECTION *frontend,
 			if (MAJOR(backend) == PROTO_MAJOR_V3)
 			{
 				char kind;
-				POOL_STATUS status;
 
 				if (pool_read(frontend, &kind, 1) < 0)
 					return POOL_END;
 				
-				status = SimpleForwardToBackend(kind, frontend, backend);
-				if (status == POOL_END)
-					return status;
+				SimpleForwardToBackend(kind, frontend, backend);
 
 				/* CopyData? */
 				if (kind == 'd')
@@ -1056,11 +1057,7 @@ static POOL_STATUS CopyDataRows(POOL_CONNECTION *frontend,
 					break;
 			}
 			else
-			{
 				string = pool_read_string(frontend, &len, 1);
-				if (string == NULL)
-					return POOL_END;
-			}
 		}
 		else
 		{
@@ -1068,14 +1065,11 @@ static POOL_STATUS CopyDataRows(POOL_CONNECTION *frontend,
 			if (MAJOR(backend) == PROTO_MAJOR_V3)
 			{
 				signed char kind;
-				POOL_STATUS status;
 
 				if ((kind = pool_read_kind(backend)) < 0)
 					return POOL_END;
 				
-				status = SimpleForwardToFrontend(kind, frontend, backend);
-				if (status == POOL_END)
-					return status;
+				SimpleForwardToFrontend(kind, frontend, backend);
 
 				/* CopyData? */
 				if (kind == 'd')
@@ -1095,16 +1089,8 @@ static POOL_STATUS CopyDataRows(POOL_CONNECTION *frontend,
 			return POOL_END;
 
 #ifdef PRINT_DEBUG
-		buf = malloc(len + 1);
-		if (buf == NULL)
-		{
-			show_error("CopyDataRows: malloc failed: %s", strerror(errno));
-			return POOL_END;
-		}
 		strncpy(buf, string, len);
-		buf[len] = '\0';
 		show_debug("%s: copy line %d %d bytes :%s:",func, i++, len, buf);
-		free(buf);
 #endif
 
 		if (copyin)
@@ -1927,10 +1913,8 @@ static int reset_backend(POOL_CONNECTION_POOL *backend, int qcnt)
 	int qn = sizeof(queries)/sizeof(char *);
 
 	/* for PGCluster */
-	if (!Use_Connection_Pool)
-		return 2;
-
-	if (qcnt >= qn)
+	if ((! Use_Connection_Pool) ||
+		(qcnt >= qn))
 		return 2;
 
 	query = queries[qcnt];
@@ -2097,3 +2081,5 @@ void pool_send_error_message(POOL_CONNECTION *frontend, int protoMajor,
 	else
 		show_error("%s: unknown protocol major %d",func, protoMajor);
 }
+
+#endif /* USE_REPLICATION */
