@@ -8,7 +8,7 @@
 #ifndef PGREPLICATE_H
 #define PGREPLICATE_H
 
-#define PGREPLICATE_VERSION		"1.1.0b"
+#define PGREPLICATE_VERSION		"1.1.1a"
 
 #if 0
 #define COMMIT_ABORT_TEST	(1)
@@ -244,7 +244,6 @@ extern int QueryLogAnsMsgid;
 extern int PGconnMsgid;
 extern int SemID;
 extern int RecoverySemID;
-extern int LockSemID;
 
 extern int MaxBackends;
 extern char * PGR_Result;
