@@ -307,13 +307,10 @@ PGRsend_load_balance_packet(RecoveryPacket * packet)
 	while (lbp->hostName[0] != 0)
 	{
 		status = send_packet(lbp,packet);
-		if (status == STATUS_OK)
+		if (lbp->recovery_sock != -1)
 		{
-			if (lbp->sock != -1)
-			{
-				close(lbp->sock);
-				lbp->sock = -1;
-			}
+			close(lbp->recovery_sock);
+			lbp->recovery_sock = -1;
 		}
 		lbp ++;
 	}
@@ -634,7 +631,7 @@ PGRsend_queue(RecoveryTbl * master, RecoveryTbl * target)
 	RecoveryQueue.queue_fp = PGRget_recovery_queue_file_for_read(0);
 	if (RecoveryQueue.queue_fp == NULL)
 	{
-		return STATUS_ERROR;
+		return STATUS_OK;
 	}
 	clearerr(RecoveryQueue.queue_fp);
 	fseek(RecoveryQueue.queue_fp,0,SEEK_SET);
@@ -711,6 +708,11 @@ PGRsend_queue(RecoveryTbl * master, RecoveryTbl * target)
 	{
 		free(query);
 	}
+	if (read_lock)
+	{
+		read_lock = 0;
+		PGRsem_unlock(RecoverySemID, SEM_NUM_OF_RECOVERY_QUEUE);
+	}
 	return status;
 }
 
@@ -833,7 +835,6 @@ pgrecovery_loop(int fd)
 				show_debug("%s:last target %s - %d",
 					func,Target.hostName,Target.port);
 #endif			
-				sleep(5);
 				status = PGRsend_queue(&Master,&Target);
 				if (status == STATUS_OK)
 				{
@@ -887,12 +888,15 @@ pgrecovery_loop(int fd)
 			if (Master.sock != -1)
 			{
 				close (Master.sock);
+				Master.sock = -1;
 			}
 			if (Master.recovery_sock != -1)
 			{
 				close (Master.recovery_sock);
+				Master.recovery_sock = -1;
 			}
-			PGR_Close_Sock(&sock);
+			close(sock);
+			sock = -1;
 			return;
 		}
 	}
@@ -1076,6 +1080,7 @@ clear_recovery_queue_file(void)
 	read_queue_no = Recovery_Status_Inf->read_queue_no;
 	if (write_queue_no != read_queue_no)
 	{
+		PGRsem_unlock(RecoverySemID, SEM_NUM_OF_RECOVERY_QUEUE);
 		return;
 	}
 	if (RecoveryQueue.queue_fp != NULL)
