@@ -841,7 +841,7 @@ PGRinit_recovery(void)
 		show_error("%s:semget() failed. (%s)",func,strerror(errno));
 		return STATUS_ERROR;
 	}
-	for ( i = 0 ; i < 2 ; i ++)
+	for ( i = 0 ; i < 4 ; i ++)
 	{
 		semctl(RecoverySemID, i, GETVAL, sem_arg);
 		sem_arg.val = 1;
@@ -1050,11 +1050,6 @@ PGRreplicate_exit(int sig)
 		sem_quit(RecoverySemID);
 		RecoverySemID = 0;
 	}
-	if (LockSemID > 0)
-	{
-		sem_quit(LockSemID);
-		LockSemID = 0;
-	}
 
 	snprintf(fname, sizeof(fname), "%s/%s", PGR_Data_Path, PGREPLICATE_PID_FILE);
 	unlink(fname);
