@@ -22,7 +22,6 @@
 
 #include <errno.h>
 
-
 #ifdef WIN32
 #include "win32.h"
 #else
@@ -44,17 +43,6 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
-#include <ctype.h>
-#include <search.h>
-
-extern char *optarg;
-extern int	optind;
-
-#ifdef WIN32
-#undef select
-#endif
-
-
 /********************************************************************
  * some configurable parameters */
 
@@ -83,10 +71,6 @@ int			tps = 1;
 #define	UPDATE_ONLY	(3)
 #define	WITH_TRANSACTION	(4)
 #define TPC_B_LIKE	(5)
-#define CUSTOM_QUERY	(6)
-
-#define SQL_COMMAND		1
-#define META_COMMAND	2
 
 FILE	   *LOGFILE = NULL;
 
@@ -105,12 +89,6 @@ char	   *login = NULL;
 char	   *pwd = NULL;
 char	   *dbName;
 
-typedef struct
-{
-	char	   *name;
-	char	   *value;
-}	Variable;
-
 typedef struct
 {
 	PGconn	   *con;			/* connection handle to DB */
@@ -126,23 +104,13 @@ typedef struct
 	int			tid;			/* teller id for this transaction */
 	int			delta;
 	int			abalance;
-	void	   *variables;
 	struct timeval txn_begin;	/* used for measuring latencies */
 }	CState;
 
-typedef struct
-{
-	int			type;
-	int			argc;
-	char	  **argv;
-}	Command;
-
-Command	  **commands = NULL;
-
 static void
-usage(void)
+usage()
 {
-	fprintf(stderr, "usage: pgcbench [-h hostname][-p port][-c nclients][-t ntransactions][-s scaling_factor][-I(insert only)][-U(update only)][-S(select only)][-f filename][-u login][-P password][-d(debug)][dbname]\n");
+	fprintf(stderr, "usage: pgcbench [-h hostname][-p port][-c nclients][-t ntransactions][-s scaling_factor][-I(insert only)][-U(update only)][-S(select only)][-u login][-P password][-d(debug)][dbname]\n");
 	fprintf(stderr, "(initialize mode): pgcbench -i [-h hostname][-p port][-s scaling_factor][-u login][-P password][-d(debug)][dbname]\n");
 }
 
@@ -156,11 +124,11 @@ getrand(int min, int max )
 
 /* set up a connection to the backend */
 static PGconn *
-doConnect(void)
+doConnect()
 {
 	PGconn	   *con;
 	PGresult   *res;
-
+ 
 	con = PQsetdbLogin(pghost, pgport, pgoptions, pgtty, dbName,
 					   login, pwd);
 	if (con == NULL)
@@ -189,6 +157,13 @@ doConnect(void)
 		exit(1);
 	}
 	PQclear(res);
+	res = PQexec(con, "SET autocommit TO 'on'");
+	if (PQresultStatus(res) != PGRES_COMMAND_OK)
+	{
+		fprintf(stderr, "%s", PQerrorMessage(con));
+		exit(1);
+	}
+	PQclear(res);
 
 	return (con);
 }
@@ -221,118 +196,6 @@ check(CState * st, PGresult *res, int good)
 	return (0);					/* OK */
 }
 
-static int
-compareVariables(const void *v1, const void *v2)
-{
-	return strcmp(((Variable *)v1)->name, ((Variable *)v2)->name);
-}
-
-static char *
-getVariable(CState * st, char *name)
-{
-	Variable		key = { name }, *var;
-
-	var = tfind(&key, &st->variables, compareVariables);
-	if (var != NULL)
-		return (*(Variable **)var)->value;
-	else
-		return NULL;
-}
-
-static int
-putVariable(CState * st, char *name, char *value)
-{
-	Variable		key = { name }, *var;
-
-	var = tfind(&key, &st->variables, compareVariables);
-	if (var == NULL)
-	{
-		if ((var = malloc(sizeof(Variable))) == NULL)
-			return false;
-
-		var->name = NULL;
-		var->value = NULL;
-
-		if ((var->name = strdup(name)) == NULL
-			|| (var->value = strdup(value)) == NULL
-			|| tsearch(var, &st->variables, compareVariables) == NULL)
-		{
-			free(var->name);
-			free(var->value);
-			free(var);
-			return false;
-		}
-	}
-	else
-	{
-		free((*(Variable **)var)->value);
-		if (((*(Variable **)var)->value = strdup(value)) == NULL)
-			return false;
-	}
-
-	return true;
-}
-
-static char *
-assignVariables(CState * st, char *sql)
-{
-	int			i, j;
-	char	   *p, *name, *val;
-	void	   *tmp;
-
-	i = 0;
-	while ((p = strchr(&sql[i], ':')) != NULL)
-	{
-		i = j = p - sql;
-		do{
-			i++;
-		}while (isalnum((unsigned char) sql[i]) != 0 || sql[i] == '_');
-		if (i == j + 1)
-			continue;
-
-		name = malloc(i - j);
-		if (name == NULL)
-			return NULL;
-		memcpy(name, &sql[j + 1], i-(j+1));
-		name[i-(j+1)] = '\0';
-		val = getVariable(st, name);
-		free(name);
-		if (val == NULL)
-			continue;
-
-		if (strlen(val) > i - j)
-		{
-			tmp = realloc(sql, strlen(sql) - (i - j) + strlen(val) + 1);
-			if (tmp == NULL)
-			{
-				free(sql);
-				return NULL;
-			}
-			sql = tmp;
-		}
-
-		if (strlen(val) != i - j)
-			memmove(&sql[j + strlen(val)], &sql[i], strlen(&sql[i]) + 1);
-
-		strncpy(&sql[j], val, strlen(val));
-
-		if (strlen(val) < i - j)
-		{
-			tmp = realloc(sql, strlen(sql) + 1);
-			if (tmp == NULL)
-			{
-				free(sql);
-				return NULL;
-			}
-			sql = tmp;
-		}
-
-		i = j + strlen(val);
-	}
-
-	return sql;
-}
-
 /* process a transaction */
 static void
 doMix(CState * st, int debug, int ttype)
@@ -414,14 +277,14 @@ doMix(CState * st, int debug, int ttype)
 				 */
 				if (use_log)
 				{
-					double		diff;
+					long long	diff;
 					struct timeval now;
 
-					gettimeofday(&now, NULL);
-					diff = (int) (now.tv_sec - st->txn_begin.tv_sec) * 1000000.0 +
-						(int) (now.tv_usec - st->txn_begin.tv_usec);
+					gettimeofday(&now, 0);
+					diff = (now.tv_sec - st->txn_begin.tv_sec) * 1000000 +
+						(now.tv_usec - st->txn_begin.tv_usec);
 
-					fprintf(LOGFILE, "%d %d %.0f\n", st->id, st->cnt, diff);
+					fprintf(LOGFILE, "%d %d %lld\n", st->id, st->cnt, diff);
 				}
 
 				res = PQgetResult(st->con);
@@ -494,7 +357,7 @@ doMix(CState * st, int debug, int ttype)
 			st->tid = getrand(1, ntellers * tps);
 			st->delta = getrand(1, 1000);
 			if (use_log)
-				gettimeofday(&(st->txn_begin), NULL);
+				gettimeofday(&(st->txn_begin), 0);
 			break;
 		case 1:
 			snprintf(sql, 256, "update accounts set abalance = abalance + %d where aid = %d\n", st->delta, st->aid);
@@ -663,172 +526,6 @@ doOne(CState * st, int debug, int ttype )
 	}
 }
 
-static void
-doCustom(CState * st, int debug, int ttype )
-{
-	PGresult   *res;
-
-	if (st->listen)
-	{							/* are we receiver? */
-		if (commands[st->state]->type == SQL_COMMAND)
-		{
-			if (debug)
-				fprintf(stderr, "client receiving\n");
-			if (!PQconsumeInput(st->con))
-			{						/* there's something wrong */
-				fprintf(stderr, "Client aborted in state %d. Probably the backend died while processing.\n", st->state);
-				PQfinish(st->con);
-				st->con = NULL;
-				return;
-			}
-			if (PQisBusy(st->con))
-				return;				/* don't have the whole result yet */
-		}
-
-		/*
-		 * transaction finished: record the time it took in the
-		 * log
-		 */
-		if (use_log && commands[st->state + 1] == NULL)
-		{
-			double		diff;
-			struct timeval now;
-
-			gettimeofday(&now, NULL);
-			diff = (int) (now.tv_sec - st->txn_begin.tv_sec) * 1000000.0 +
-				(int) (now.tv_usec - st->txn_begin.tv_usec);
-
-			fprintf(LOGFILE, "%d %d %.0f\n", st->id, st->cnt, diff);
-		}
-
-		if (commands[st->state]->type == SQL_COMMAND)
-		{
-			res = PQgetResult(st->con);
-			if (strncasecmp(commands[st->state]->argv[0], "select", 6) != 0)
-			{
-				if (check(st, res, PGRES_COMMAND_OK))
-					return;
-			}
-			else
-			{
-				if (check(st, res, PGRES_TUPLES_OK))
-					return;
-			}
-			PQclear(res);
-			discard_response(st);
-		}
-
-		if (commands[st->state + 1] == NULL)
-		{
-			if (is_connect)
-			{
-				PQfinish(st->con);
-				st->con = NULL;
-			}
-			if (++st->cnt >= st->maxAct)
-			{
-				remains--;			/* I've done */
-				if (st->con != NULL)
-				{
-					PQfinish(st->con);
-					st->con = NULL;
-				}
-				return;
-			}
-		}
-
-		/* increment state counter */
-		st->state++;
-		if (commands[st->state] == NULL)
-		{
-			st->state = 0;
-			remains--;			/* I've done */
-		}
-	}
-
-	if (st->con == NULL)
-	{
-		if ((st->con = doConnect()) == NULL)
-		{
-			fprintf(stderr, "Client aborted in establishing connection.\n");
-			remains--;			/* I've aborted */
-			PQfinish(st->con);
-			st->con = NULL;
-			return;
-		}
-	}
-
-	if (use_log && st->state == 0)
-		gettimeofday(&(st->txn_begin), NULL);
-
-	if (commands[st->state]->type == SQL_COMMAND)
-	{
-		char	   *sql;
-
-		if ((sql = strdup(commands[st->state]->argv[0])) == NULL
-			|| (sql = assignVariables(st, sql)) == NULL)
-		{
-			fprintf(stderr, "out of memory\n");
-			st->ecnt++;
-			return;
-		}
-
-		if (debug)
-			fprintf(stderr, "client sending %s\n", sql);
-
-		if (PQsendQuery(st->con, sql) == 0)
-		{
-			if (debug)
-				fprintf(stderr, "PQsendQuery(%s)failed\n", sql);
-			st->ecnt++;
-		}
-		else
-		{
-			st->listen++;			/* flags that should be listened */
-		}
-
-		free(sql);
-	}
-	else if (commands[st->state]->type == META_COMMAND)
-	{
-		int			argc = commands[st->state]->argc, i;
-		char	  **argv = commands[st->state]->argv;
-
-		if (debug)
-		{
-			fprintf(stderr, "client executing \\%s", argv[0]);
-			for (i = 1; i < argc; i++)
-				fprintf(stderr, " %s", argv[i]);
-			fprintf(stderr, "\n");
-		}
-
-		if (strcasecmp(argv[0], "setrandom") == 0)
-		{
-			char	   *val;
-
-			if ((val = malloc(strlen(argv[3]) + 1)) == NULL)
-			{
-				fprintf(stderr, "%s: out of memory\n", argv[0]);
-				st->ecnt++;
-				return;
-			}
-
-			sprintf(val, "%d", getrand(atoi(argv[2]), atoi(argv[3])));
-
-			if (putVariable(st, argv[1], val) == false)
-			{
-				fprintf(stderr, "%s: out of memory\n", argv[0]);
-				free(val);
-				st->ecnt++;
-				return;
-			}
-
-			free(val);
-			st->listen++;
-		}
-	}
-}
-
 /* discard connections */
 static void
 disconnect_all(CState * state)
@@ -845,19 +542,13 @@ init(void)
 	PGresult   *res;
 	static char *DDLs[] = {
 		"drop table branches",
-		"create table branches(bid int not null,bbalance int,filler char(88))",
+		"create table branches(bid int, primary key(bid),bbalance int,filler char(88))",
 		"drop table tellers",
-		"create table tellers(tid int not null,bid int,tbalance int,filler char(84))",
+		"create table tellers(tid int, primary key(tid),bid int,tbalance int,filler char(84))",
 		"drop table accounts",
-		"create table accounts(aid int not null,bid int,abalance int,filler char(84))",
+		"create table accounts(aid int,primary key(aid),bid int,abalance int,filler char(84))",
 		"drop table history",
 	"create table history(tid int,bid int,aid int,delta int,mtime timestamp,filler char(22))"};
-	static char *DDLAFTERs[] = {
-		"alter table branches add primary key (bid)",
-		"alter table tellers add primary key (tid)",
-	"alter table accounts add primary key (aid)"};
-
-
 	char		sql[256];
 
 	int			i;
@@ -915,7 +606,6 @@ init(void)
 		fprintf(stderr, "%s", PQerrorMessage(con));
 		exit(1);
 	}
-	PQclear(res);
 
 	/*
 	 * occupy accounts table with some data
@@ -936,7 +626,7 @@ init(void)
 			PQclear(res);
 		}
 
-		snprintf(sql, 256, "%d\t%d\t%d\t\n", j, i / naccounts + 1, 0);
+		snprintf(sql, 256, "%d\t%d\t%d\t\n", j, j / naccounts, 0);
 		if (PQputline(con, sql))
 		{
 			fprintf(stderr, "PQputline failed\n");
@@ -977,18 +667,6 @@ init(void)
 #endif   /* NOT_USED */
 		}
 	}
-	fprintf(stderr, "set primary key...\n");
-	for (i = 0; i < (sizeof(DDLAFTERs) / sizeof(char *)); i++)
-	{
-		res = PQexec(con, DDLAFTERs[i]);
-		if (PQresultStatus(res) != PGRES_COMMAND_OK)
-		{
-			fprintf(stderr, "%s", PQerrorMessage(con));
-			exit(1);
-		}
-		PQclear(res);
-	}
-
 	/* vacuum */
 	fprintf(stderr, "vacuum...");
 	res = PQexec(con, "vacuum analyze");
@@ -1003,160 +681,6 @@ init(void)
 	PQfinish(con);
 }
 
-static int
-process_file(char *filename)
-{
-	const char	delim[] = " \f\n\r\t\v";
-
-	FILE	   *fd;
-	int			lineno, i, j;
-	char		buf[BUFSIZ], *p, *tok;
-	void	   *tmp;
-
-	if (strcmp(filename, "-") == 0)
-		fd = stdin;
-	else if ((fd = fopen(filename, "r")) == NULL)
-	{
-		fprintf(stderr, "%s: %s\n", strerror(errno), filename);
-		return false;
-	}
-
-	fprintf(stderr, "processing file...\n");
-
-	lineno = 1;
-	i = 0;
-	while (fgets(buf, sizeof(buf), fd) != NULL)
-	{
-		if ((p = strchr(buf, '\n')) != NULL)
-			*p = '\0';
-		p = buf;
-		while (isspace((unsigned char)*p))
-			p++;
-		if (*p == '\0' || strncmp(p, "--", 2) == 0)
-		{
-			lineno++;
-			continue;
-		}
-
-		if ((tmp = realloc(commands, sizeof(Command *) * (i + 1))) == NULL)
-		{
-			i--;
-			goto error;
-		}
-		commands = tmp;
-
-		if ((commands[i] = malloc(sizeof(Command))) == NULL)
-			goto error;
-
-		commands[i]->argv = NULL;
-		commands[i]->argc = 0;
-
-		if (*p == '\\')
-		{
-			commands[i]->type = META_COMMAND;
-
-			j = 0;
-			tok = strtok(++p, delim);
-			while (tok != NULL)
-			{
-				tmp = realloc(commands[i]->argv, sizeof(char *) * (j + 1));
-				if (tmp == NULL)
-					goto error;
-				commands[i]->argv = tmp;
-
-				if ((commands[i]->argv[j] = strdup(tok)) == NULL)
-					goto error;
-
-				commands[i]->argc++;
-
-				j++;
-				tok = strtok(NULL, delim);
-			}
-
-			if (strcasecmp(commands[i]->argv[0], "setrandom") == 0)
-			{
-				int			min, max;
-
-				if (commands[i]->argc < 4)
-				{
-					fprintf(stderr, "%s: %d: \\%s: missing argument\n", filename, lineno, commands[i]->argv[0]);
-					goto error;
-				}
-
-				for (j = 4; j < commands[i]->argc; j++)
-					fprintf(stderr, "%s: %d: \\%s: extra argument \"%s\" ignored\n", filename, lineno, commands[i]->argv[0], commands[i]->argv[j]);
-
-				if ((min = atoi(commands[i]->argv[2])) < 0)
-				{
-					fprintf(stderr, "%s: %d: \\%s: invalid minimum number %s\n", filename, lineno, commands[i]->argv[0], commands[i]->argv[2]);
-					goto error;
-				}
-
-				if ((max = atoi(commands[i]->argv[3])) < min || max > RAND_MAX)
-				{
-					fprintf(stderr, "%s: %d: \\%s: invalid maximum number %s\n", filename, lineno, commands[i]->argv[0], commands[i]->argv[3]);
-					goto error;
-				}
-			}
-			else
-			{
-				fprintf(stderr, "%s: %d: invalid command \\%s\n", filename, lineno, commands[i]->argv[0]);
-				goto error;
-			}
-		}
-		else
-		{
-			commands[i]->type = SQL_COMMAND;
-
-			if ((commands[i]->argv = malloc(sizeof(char *))) == NULL)
-				goto error;
-
-			if ((commands[i]->argv[0] = strdup(p)) == NULL)
-				goto error;
-
-			commands[i]->argc++;
-		}
-
-		i++;
-		lineno++;
-	}
-	fclose(fd);
-
-	if ((tmp = realloc(commands, sizeof(Command *) * (i + 1))) == NULL)
-		goto error;
-	commands = tmp;
-
-	commands[i] = NULL;
-
-	return true;
-
-error:
-	if (errno == ENOMEM)
-		fprintf(stderr, "%s: %d: out of memory\n", filename, lineno);
-
-	fclose(fd);
-
-	if (commands == NULL)
-		return false;
-
-	while (i >= 0)
-	{
-		if (commands[i] != NULL)
-		{
-			for (j = 0; j < commands[i]->argc; j++)
-				free(commands[i]->argv[j]);
-
-			free(commands[i]->argv);
-			free(commands[i]);
-		}
-
-		i--;
-	}
-	free(commands);
-
-	return false;
-}
-
 /* print out results */
 static void
 printResults(
@@ -1192,9 +716,6 @@ printResults(
 		case UPDATE_ONLY :
 			s = "UPDATE only";
 			break;
-		case CUSTOM_QUERY :
-			s = "Custom query";
-			break;
 		default:
 			s = "Mix query";
 			break;
@@ -1219,8 +740,8 @@ doChild(int clientId, int min, int max, int debug, int ttype)
 	int			nsocks = 0;		/* return from select(2) */
 	int			sock = 0;
 
-	gettimeofday(&tv1, NULL);
-	srand((unsigned int) tv1.tv_usec + clientId );
+	gettimeofday(&tv1, 0);
+	srand((uint) tv1.tv_usec + clientId );
 
 	memset((char *)&state,0,sizeof(CState));
 	/* make connections to the database */
@@ -1236,9 +757,6 @@ doChild(int clientId, int min, int max, int debug, int ttype)
 		case TPC_B_LIKE :
 			doMix(&state, debug, ttype);
 			break;
-		case CUSTOM_QUERY :
-			doCustom(&state, debug, ttype);
-			break;
 		default :
 			doOne(&state, debug, ttype);
 			break;
@@ -1247,55 +765,50 @@ doChild(int clientId, int min, int max, int debug, int ttype)
 	remains = max;
 	for (;;)
 	{
-		if (remains < min || !state.con)
+		if (remains < min)
 		{
 			break;
 		}
 
 		FD_ZERO(&input_mask);
 
-		if (ttype != CUSTOM_QUERY || commands[state.state]->type != META_COMMAND)
+		if (state.con == NULL)
 		{
-			if (state.con == NULL)
+			if ((state.con = doConnect()) == NULL)
 			{
-				if ((state.con = doConnect()) == NULL)
-				{
-					exit(1);
-				}
-			}
-			sock = PQsocket(state.con);
-
-			if (sock < 0)
-			{
-				fprintf(stderr, "Client %d: PQsocket failed\n", clientId);
-				disconnect_all(&state);
 				exit(1);
 			}
-			FD_SET(sock, &input_mask);
+		}
+		sock = PQsocket(state.con);
 
-			if ((nsocks = select(sock + 1, &input_mask, (fd_set *) NULL,
-							  (fd_set *) NULL, (struct timeval *) NULL)) < 0)
-			{
-				if (errno == EINTR)
-					continue;
-				/* must be something wrong */
-				disconnect_all(&state);
-				fprintf(stderr, "select failed: %s\n", strerror(errno));
-				exit(1);
-			}
-			else if (nsocks == 0)
-			{						/* timeout */
-				fprintf(stderr, "select timeout\n");
-				fprintf(stderr, "client %d:state %d cnt %d ecnt %d listen %d\n",
-						clientId, state.state, state.cnt, state.ecnt, state.listen);
-				exit(0);
-			}
+		if (sock < 0)
+		{
+			fprintf(stderr, "Client %d: PQsocket failed\n", clientId);
+			disconnect_all(&state);
+			exit(1);
+		}
+		FD_SET(sock, &input_mask);
+
+		if ((nsocks = select(sock + 1, &input_mask, (fd_set *) NULL,
+						  (fd_set *) NULL, (struct timeval *) NULL)) < 0)
+		{
+			if (errno == EINTR)
+				continue;
+			/* must be something wrong */
+			disconnect_all(&state);
+			fprintf(stderr, "select failed: %s\n", strerror(errno));
+			exit(1);
+		}
+		else if (nsocks == 0)
+		{						/* timeout */
+			fprintf(stderr, "select timeout\n");
+			fprintf(stderr, "client %d:state %d cnt %d ecnt %d listen %d\n",
+					clientId, state.state, state.cnt, state.ecnt, state.listen);
+			exit(0);
 		}
 
 		/* ok, backend returns reply */
-		if (state.con && (FD_ISSET(PQsocket(state.con), &input_mask)
-						  || (ttype == CUSTOM_QUERY
-							  && commands[state.state]->type == META_COMMAND)))
+		if (state.con && FD_ISSET(PQsocket(state.con), &input_mask))
 		{
 			switch (ttype)
 			{
@@ -1303,9 +816,6 @@ doChild(int clientId, int min, int max, int debug, int ttype)
 				case TPC_B_LIKE :
 					doMix(&state, debug, ttype);
 					break;
-				case CUSTOM_QUERY :
-					doCustom(&state, debug, ttype);
-					break;
 				default :
 					doOne(&state, debug, ttype);
 					break;
@@ -1351,6 +861,8 @@ doClient(int debug, int ttype)
 int
 main(int argc, char **argv)
 {
+	extern char *optarg;
+	extern int	optind;
 	int			c;
 	int			is_init_mode = 0;		/* initialize mode? */
 	int			is_no_vacuum = 0;		/* no vacuum at all before
@@ -1358,29 +870,20 @@ main(int argc, char **argv)
 	int			is_full_vacuum = 0;		/* do full vacuum before testing? */
 	int			debug = 0;		/* debug flag */
 	int			ttype = TPC_B_LIKE;		/* transaction type */
-	char	   *filename = NULL;
 
 	struct timeval tv1;			/* start up time */
 	struct timeval tv2;			/* after establishing all connections to
 								 * the backend */
 	struct timeval tv3;			/* end time */
 
-#if !(defined(__CYGWIN__) || defined(__MINGW32__))
+#ifndef __CYGWIN__
 	struct rlimit rlim;
 #endif
 
 	PGconn	   *con;
 	PGresult   *res;
-	char	   *env;
 
-	if ((env = getenv("PGHOST")) != NULL && *env != '\0')
-		pghost = env;
-	if ((env = getenv("PGPORT")) != NULL && *env != '\0')
-		pgport = env;
-	else if ((env = getenv("PGUSER")) != NULL && *env != '\0')
-		login = env;
-
-	while ((c = getopt(argc, argv, "ih:nvp:dc:t:s:u:P:CNSlTUIf:")) != -1)
+	while ((c = getopt(argc, argv, "ih:nvp:dc:t:s:u:P:CNSlTUI")) != -1)
 	{
 		switch (c)
 		{
@@ -1421,7 +924,7 @@ main(int argc, char **argv)
 					fprintf(stderr, "invalid number of clients: %d\n", nclients);
 					exit(1);
 				}
-#if !(defined(__CYGWIN__) || defined(__MINGW32__))
+#ifndef __CYGWIN__
 #ifdef RLIMIT_NOFILE			/* most platform uses RLIMIT_NOFILE */
 				if (getrlimit(RLIMIT_NOFILE, &rlim) == -1)
 				{
@@ -1438,7 +941,7 @@ main(int argc, char **argv)
 					fprintf(stderr, "Use limit/ulimt to increase the limit before using pgbench.\n");
 					exit(1);
 				}
-#endif   /* #if !(defined(__CYGWIN__) || defined(__MINGW32__)) */
+#endif   /* #ifndef __CYGWIN__ */
 				break;
 			case 'C':
 				is_connect = 1;
@@ -1468,10 +971,6 @@ main(int argc, char **argv)
 			case 'l':
 				use_log = true;
 				break;
-			case 'f':
-				ttype = CUSTOM_QUERY;
-				filename = optarg;
-				break;
 			default:
 				usage();
 				exit(1);
@@ -1483,11 +982,8 @@ main(int argc, char **argv)
 		dbName = argv[optind];
 	else
 	{
-		if ((env = getenv("PGDATABASE")) != NULL && *env != '\0')
-			dbName = env;
-		else if (login != NULL && *login != '\0')
-			dbName = login;
-		else
+		dbName = getenv("USER");
+		if (dbName == NULL)
 			dbName = "";
 	}
 
@@ -1529,96 +1025,87 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
-	if (ttype == CUSTOM_QUERY)
+	/*
+	 * get the scaling factor that should be same as count(*) from
+	 * branches...
+	 */
+	res = PQexec(con, "select count(*) from branches");
+	if (PQresultStatus(res) != PGRES_TUPLES_OK)
 	{
-		PQfinish(con);
-		if (process_file(filename) == false)
-			exit(1);
+		fprintf(stderr, "%s", PQerrorMessage(con));
+		exit(1);
 	}
-	else
+	tps = atoi(PQgetvalue(res, 0, 0));
+	if (tps < 0)
 	{
-		/*
-		 * get the scaling factor that should be same as count(*) from
-		 * branches...
-		 */
-		res = PQexec(con, "select count(*) from branches");
-		if (PQresultStatus(res) != PGRES_TUPLES_OK)
+		fprintf(stderr, "count(*) from branches invalid (%d)\n", tps);
+		exit(1);
+	}
+	PQclear(res);
+
+	if (!is_no_vacuum)
+	{
+		fprintf(stderr, "starting vacuum...");
+		res = PQexec(con, "vacuum branches");
+		if (PQresultStatus(res) != PGRES_COMMAND_OK)
 		{
 			fprintf(stderr, "%s", PQerrorMessage(con));
 			exit(1);
 		}
-		tps = atoi(PQgetvalue(res, 0, 0));
-		if (tps < 0)
+		PQclear(res);
+
+		res = PQexec(con, "vacuum tellers");
+		if (PQresultStatus(res) != PGRES_COMMAND_OK)
 		{
-			fprintf(stderr, "count(*) from branches invalid (%d)\n", tps);
+			fprintf(stderr, "%s", PQerrorMessage(con));
 			exit(1);
 		}
 		PQclear(res);
 
-		if (!is_no_vacuum)
+		res = PQexec(con, "delete from history");
+		if (PQresultStatus(res) != PGRES_COMMAND_OK)
 		{
-			fprintf(stderr, "starting vacuum...");
-			res = PQexec(con, "vacuum branches");
-			if (PQresultStatus(res) != PGRES_COMMAND_OK)
-			{
-				fprintf(stderr, "%s", PQerrorMessage(con));
-				exit(1);
-			}
-			PQclear(res);
+			fprintf(stderr, "%s", PQerrorMessage(con));
+			exit(1);
+		}
+		PQclear(res);
+		res = PQexec(con, "vacuum history");
+		if (PQresultStatus(res) != PGRES_COMMAND_OK)
+		{
+			fprintf(stderr, "%s", PQerrorMessage(con));
+			exit(1);
+		}
+		PQclear(res);
 
-			res = PQexec(con, "vacuum tellers");
-			if (PQresultStatus(res) != PGRES_COMMAND_OK)
-			{
-				fprintf(stderr, "%s", PQerrorMessage(con));
-				exit(1);
-			}
-			PQclear(res);
+		fprintf(stderr, "end.\n");
 
-			res = PQexec(con, "delete from history");
+		if (is_full_vacuum)
+		{
+			fprintf(stderr, "starting full vacuum...");
+			res = PQexec(con, "vacuum analyze accounts");
 			if (PQresultStatus(res) != PGRES_COMMAND_OK)
 			{
 				fprintf(stderr, "%s", PQerrorMessage(con));
 				exit(1);
 			}
 			PQclear(res);
-			res = PQexec(con, "vacuum history");
-			if (PQresultStatus(res) != PGRES_COMMAND_OK)
-			{
-				fprintf(stderr, "%s", PQerrorMessage(con));
-				exit(1);
-			}
-			PQclear(res);
-
 			fprintf(stderr, "end.\n");
-
-			if (is_full_vacuum)
-			{
-				fprintf(stderr, "starting full vacuum...");
-				res = PQexec(con, "vacuum analyze accounts");
-				if (PQresultStatus(res) != PGRES_COMMAND_OK)
-				{
-					fprintf(stderr, "%s", PQerrorMessage(con));
-					exit(1);
-				}
-				PQclear(res);
-				fprintf(stderr, "end.\n");
-			}
 		}
-		PQfinish(con);
 	}
+	PQfinish(con);
 	
 	/* set random seed */
-	gettimeofday(&tv1, NULL);
-	srand((unsigned int) tv1.tv_usec);
+	gettimeofday(&tv1, 0);
+	srand((uint) tv1.tv_usec);
 	/* get start up time */
-	gettimeofday(&tv1, NULL);
+	gettimeofday(&tv1, 0);
 	/* time after connections set up */
-	gettimeofday(&tv2, NULL);
+	gettimeofday(&tv2, 0);
 
 	doClient(debug, ttype);
 
 	/* get end time */
-	gettimeofday(&tv3, NULL);
+	gettimeofday(&tv3, 0);
 	printResults(ttype, nxacts, &tv1, &tv2, &tv3);
 	if (LOGFILE)
 		fclose(LOGFILE);
